﻿using System.Linq;
using System.Windows.Media;
using System.ComponentModel;

namespace ININ.Supervisor.Chart
{
    public class ChartStatistic : INotifyPropertyChanged
    {
        private readonly string _StatisticId;
        private readonly string _StatisticDisplayString;
        private double _Value = 0.0;
        private object _Lock = new object();
        private int _TickCounter = -1;
        private AutoQueue<double> _60ValuesQueue = new AutoQueue<double>(60);
        private AutoQueue<double> _300SecondsQueue = new AutoQueue<double>(300);
        private AutoQueue<double> _30MinutesQueue = new AutoQueue<double>(300);
        private AutoQueue<double> _4HoursQueue = new AutoQueue<double>(300);
        private AutoQueue<double> _8HoursQueue = new AutoQueue<double>(300);
        private AutoQueue<double> _24HoursQueue = new AutoQueue<double>(300);

        public ChartStatistic(string statisticId, string statisticDisplayString)
        {
            _StatisticId = statisticId;
            _StatisticDisplayString = statisticDisplayString;
        }

        public AutoQueue<double> Queue300Seconds { get { return _300SecondsQueue; } }
        public AutoQueue<double> Queue30Minutes { get { return _30MinutesQueue; } }
        public AutoQueue<double> Queue4Hours { get { return _4HoursQueue; } }
        public AutoQueue<double> Queue8Hours { get { return _8HoursQueue; } }
        public AutoQueue<double> Queue24Hours { get { return _24HoursQueue; } }

        public string StatisticDisplayString { get { return _StatisticDisplayString; } }
        public string StatisticId { get { return _StatisticId; } }

        public double Value
        {
            get { lock (_Lock) { return _Value; } }
            set 
            { 
                lock (_Lock) 
                { 
                    _Value = value;
                    PropertyChangedEventHandler propertyChanged = PropertyChanged;
                    if (propertyChanged != null)
                        propertyChanged(this, new PropertyChangedEventArgs("Value"));
                } 
            }
        }

        private Brush _Fill;
        public Brush Fill
        {
            get { return _Fill; }
            set 
            { 
                _Fill = value;
                PropertyChangedEventHandler propertyChanged = PropertyChanged;
                if (propertyChanged != null)
                    propertyChanged(this, new PropertyChangedEventArgs("Fill"));
            }
        }

        public void Snapshot()
        {
            lock(_Lock)
            {
                ++_TickCounter;
                _60ValuesQueue.Enqueue(_Value);
                _300SecondsQueue.Enqueue(_Value);
                if (_TickCounter % 6 == 0)
                {
                    _30MinutesQueue.Enqueue(_Value);
                    if (_TickCounter % 48 == 0)
                    {
                        _4HoursQueue.Enqueue(_Value);
                        if (_TickCounter % 96 == 0)
                        {
                            _8HoursQueue.Enqueue(_Value);
                            if (_TickCounter % 288 == 0)
                                _24HoursQueue.Enqueue(_Value);
                        }
                    }
                }
            }            
        }

        public double ThrottledMaximum
        {
            get
            {
                return _60ValuesQueue.Max();
            }
        }

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion
    }
}
